
// les composants du gestionnaire : aiguille, zone, signal, autorisation, itineraire, ...

/////////////////////////////////////////////////////////////////////////////
// classe AIGUILLE

class Aiguille { 
protected:
  boolean etat=true; // directe (true) ou deviee (false)
public: 
  int no;       // numéro de l'aiguille pour commander l'electronique (moteur)
  char* nom;    // le nom de l'aiguille   OPTION nom
  Zone* zone;   // la Zone ou est l'aiguille   OPTION zone
  
  Aiguille(int n) { no=n; } // constructeur mini
  Aiguille(char* nm,int n) { nom=nm; no=n; } // constructeur maxi
  Aiguille(char* nm,int n,Zone* z) { nom=nm; no=n; zone=z; } // constructeur OPTION

  boolean directe() { return etat; }  // méthode de test
  boolean deviee()  { return !etat; } // méthode de test
  
  boolean directer(); // mettre l'aiguille en position directe
  boolean devier();   // mettre l'aiguille en position deviee

  boolean manoeuvrable(); // OPTION zone
  boolean manoeuvrer(); // commande effective de l'aiguille
  
  void init1(boolean e) { etat=e; } // initialisation etat
  void init2(boolean e) { manoeuvrer(); etat=e; } // mise en position initiale
  
  //void erreur(); // erreur sur les aiguilles (tranche 100)
};

/////////////////////////////////////////////////////////////////////////////
// classe ZONE

class Zone { 
protected:
   boolean etat=false; // libre (false) ou occupé (true)
   byte trace=0; // pour tco   OPTION
public:
   int no; // le numero de la zone pour commander l'electronique ( TCO, relais, ... )
   char* nom; // le nom de la zone   OPTION
   int type;  // caracteristiques de la zone   OPTION
   Signal* signalPair; Signal* signalImpair; // les signaux eventuels de la zone   OPTION
   Train* train=NULL; // le train dans la zone
protected:   
   //Zone(int n) { no=n; } // constructeur mini
   //Zone(char* nm,int t,int n) { nom=nm, type=t; no=n; } // constructeur maxi
   Zone(int n,Signal* sp,Signal* si); // constructeur   OPTION
 
public:   
   boolean occupee() { return etat; }  // méthode de test
   boolean libre()   { return !etat; } // méthode de test

   virtual Zone* suivantePaire()   { return NULL; } // la zone suivante paire (éventuellement vide)   OPTION
   virtual Zone* suivanteImpaire() { return NULL; } // la zone suivante impaire (éventuellement vide)   OPTION

   virtual void actions() {} // les actions spécifiques à faire en cas d'occupation
   virtual void desactions() {} // les actions spécifiques à faire en cas de libération

   void occuper(); // appelée par la rétrosignalisation lors de l'occupation d'une zone
   void liberer(); // appelée par la rétrosignalisation lors de la liberation d'une zone
   
  void init(boolean e) { etat=e; }
  
  void init() { trace=0; etat=false; } // (re)initialisation   TCO

   void tracer(boolean b); // TCO

// méthodes utilitaires

   Zone* selonAiguille(Aiguille* a,Zone* z1,Zone* z2);
   
   Zone* provenance(); // de quelle zone le train vient lors d'une occupation de zone   OPTION
   virtual Zone* ambiguite() { erreur(201); return NULL; }   
   
   void erreur(int n); // erreur sur les zones (tranche 200)
};

/////////////////////////////////////////////////////////////////////////////
// classe SIGNAL

class Signal {
protected:
  typeFeux feux; 
public:
  int no; // numero du signal
  //char* nom; // le nom du signal   OPTION
  //int type; // le type du signal   OPTION
  Zone* zone=NULL; // la zone ou est le signal
   
  Signal(typeFeux f,int n) { feux=f; no=n; } // constructeur mini
  //Signal(int t,typeFeux f,int n) { feux=f; no=n; } // constructeur 
  //Signal(char* nm,int t,typeFeux f,int n) { nom=nm; type=t; feux=f; no=n; } // constructeur maxi
  
  typeFeux getFeux() { return feux; } // methode d'acces    
  
  virtual int cibles() { return ouvert()?RIEN:CIBLE; } // les cibles a presenter   OPTION
  
  boolean ferme() { return feux&(S|C|Cv); } // methode de test
  boolean ouvert() { return !ferme(); }     // methode de test
  
  virtual void ouvrir()  {} // ouverture manuelle 
  virtual void fermer()  {} // fermeture manuelle 
  virtual void aubiner() { fermer(); } // fermeture automatique par retrosignalisation
  
  void manoeuvrer(); // manoeuvre effective du signal 

  virtual boolean ralentissement30() { return false; } // true si ralentissement a 30
  virtual boolean ralentissement60() { return false; } // true si ralentissement a 60

  virtual void majFeux() {} // mise a jour du signal precedent
  
  virtual Signal* suivant()   { return NULL; } // le signal suivant  
  virtual Signal* precedent() { return NULL; } // le signal precedent   OPTION
  
  void erreur(int n); // erreur sur les signaux (tranche 300)
  
// methodes utilitaires
  //boolean clignotant() { return (feux&(Rc+RRc)); } // methode utilitaire   OPTION
  
  Signal* selonAiguille(Aiguille* a,Signal* s1,Signal* s2) { return a->directe()?s1:s2; } 
protected:  
  void manoeuvreEtMajFeux(); // manoeuvre le signal puis fait la mise a jour du precedent
};

/////////////////////////////////////////////////////////////////////////////
// classe SIGNAL_BAL

class SignalBAL:public Signal {
protected:
  boolean modePermanent=false;   
 
  SignalBAL(typeFeux f,int n):Signal(f,n) {} // constructeur mini
  //SignalBAL(int t,typeFeux f,int n):Signal(t,f,n) {} // constructeur 
  //SignalBAL(char* nm,int t,typeFeux f,int n):Signal(nm,t,f,n) {} // constructeur maxi
public:
  virtual void aubiner() {} // fermeture automatique par retrosignalisation
  virtual void desaubiner() {} // ouverture automatique par retrosignalisation   

  void setPermanent(boolean b) { modePermanent=b; } // controle de l'ouverture permanente
  
  virtual boolean cantonOccupe() { return true; } // test d'occupation du "canton" 
};

/////////////////////////////////////////////////////////////////////////////
// classe AUTORISATION

class Autorisation { // classe de base des autorisations
protected:
  boolean sens=false; // pair=false   impair=true
  boolean prise=false; // verrou

  Autorisation() {} // constructeur vide
public:
  boolean getSens() { return sens; }
  boolean prenable(boolean s); // sens pair ou impair
  void prendre(boolean s) { sens=s; prise=true; } // sens pair ou impair
  void rendre() { prise=false; }

  virtual boolean aucunTrain() { return false; }
};

/////////////////////////////////////////////////////////////////////////////
// clase ITINERAIRE

class Itineraire { // classe de base des itineraires
protected:
  byte etat=ITINERAIRE_LIBRE; // l'etat de l'itineraire (libre, en attente, forme, ... )
  byte no=0; // numero de bouton itineraire   TCO

  Itineraire(byte n) { no=n; } // constructeur
public:
  boolean libre()     { return etat==ITINERAIRE_LIBRE || etat==ITINERAIRE_EN_ATTENTE; }
  boolean forme()     { return etat==ITINERAIRE_FORME; }
  boolean enAttente() { return etat==ITINERAIRE_EN_ATTENTE; }

  virtual boolean formable();
  virtual void former();
  virtual boolean deformable();
  virtual void deformer();
  virtual void tracer(boolean b) {} // TCO

  boolean formation();   // essai de formation (manuelle ou en memoire)
  boolean destruction(); // essai de destruction (manuelle)
  void detruire(); // destruction (appelle par les desactions() de zone ou annulation memorisation)
  void effacer();  // effacement d'un itineraire memorise // OPTION
};

/////////////////////////////////////////////////////////////////////////////

