// version 4.1 decembre 2016

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

import processing.serial.*;

int NO_PORT=2; // n° du port USB/serie utilise pour communiquer avec le gestionnaire (Arduino)

boolean points=false; // affichage des points du train pour mise au point
boolean trains=true; // presence des trains

final int N=5; // grossissement (zoom)
final int B=10; // bordure du tco en pixels
final float R=0.33F; // retrait pour separation des zones

final int L=10; // nb lignes du tco
final int C=20; // nb colonnes tco

boolean grille=true; // grille ou pas
boolean infos=false; // noms des zones, aiguilles et signaux ou pas

int DELAI=25; // pour la vitesse du train 25

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void mouseClicked() { int x,y; int l,c; Case p; // boutons
  x=mouseX; y=mouseY; c=(x-B)/(9*N); l=(y-B)/(9*N); 
  //println(x+" "+y+"   "+l+" "+c);
  if (l>=L || c>=C) return; // hors grille
  p=(tco.paves[c][l]);
  if (p==null) return; // case vide
  p.exec();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Tco tco; Serial port; Train train1,train2;
CabSignal cab1,cab2;
CabSignal[] cabs;


void setup() { 
  print("Les ports serie : ");
  println((Object[])Serial.list());
  println("Le port utilise : "+Serial.list()[NO_PORT]);
  port=new Serial(this,Serial.list()[NO_PORT],9600); 
  port.buffer(3);
  portWrite('X'); // (re)initialisation du programme de gestion
  size(1100,500,P2D); noStroke();
    
  tco=new Tco(); tco.construire(); tco.calculPoints();
  tco.dessiner();  // dessiner le TCO

  train1=new Train(1,IMPAIR,tco.z0,2,0,tco.bs2,tco.bma2); // train bleu    n° 1   4 4
  train2=new Train(2,PAIR,  tco.z1,0,0,tco.bs1,tco.bma1); // fleche d'or   n° 2
  if (!infos) { tco.bz0.train(1); tco.bz1.train(2); } // nom des trains

  cab1=new CabSignal(B+B+C*N*9,B,#FFFF77); //#FFFF6B);       // fleche d'or   n° 2
  cab2=new CabSignal(B+B+C*N*9,B+B+220,#7777FF); //#77B5FE); // train bleu    n° 1
  cab1.feux=cab2.feux=FEU_C; //Feux.CARRE;
  cabs=new CabSignal[2]; cabs[0]=cab2; cabs[1]=cab1;     // confusions avec les numeros !!!
  delay(100); 
  oldTimeTrain=oldTimeCli=millis();
}

long oldTimeTrain,oldTimeCli,newTime; 

int CLI=500;  boolean bscCli=false; // pour clignotements (boutons, feux, ...)

boolean ini=true; // pour gestion des initialisations

void draw() { 
  tco.dessiner(); // dessiner le TCO
  cab1.dessiner(); cab2.dessiner(); // dessiner les cabsignaux
  if (points) affichagePoints(); // affichage des points pour tests
  newTime=millis(); 
  if (ini && trains) { portWrite('A'); portWrite('B'); ini=false; delay(100); } // la premiere fois
  if ((newTime-oldTimeCli)>=CLI) { bscCli=!bscCli; oldTimeCli=newTime; }
  if (trains) if ((newTime-oldTimeTrain)>=DELAI) { oldTimeTrain=newTime; train1.avancer(); train2.avancer(); }
  if (trains) { train1.dessiner(COULEUR_BLEUE); train2.dessiner(COULEUR_OR); stroke(0); }
  
  // test si commandes
  if (port.available()>=3) { int code,info=0,numero; // 3 bytes   =0???
    //x=port.readChar(); code=x>>4; info=x&15; 
    code=port.readChar();  numero=port.readChar();  
    info=port.readChar();  
    //print("["+code+" "+info+" "+numero+"] ");
    affichage(code,info,numero); // affichage lisible
    // traitement des codes
    switch (code) {
      case CODE_AIGUILLE:     if (numero<2) tco.aiguilles[numero].manoeuvrer(); break; // 0 ou 1
      case CODE_ZONE:         if (numero<6) tco.zones[numero].colorer(mefCol(info)); break; // 0 a 5
      case CODE_SIGNAL:       if (numero-1<2) tco.signaux[numero-1].feu(mefFeu(info)); break; // 1 a n
      case CODE_CARRE:        if (numero-1<6) tco.carres[numero-1].feu(mefFeu(info)); break;  // 1 a n
      case CODE_BOUTON:       if (numero<8) tco.itineraires[numero].couleur(mefCol(info),false); break;
      case CODE_BOUTON_CLI:   if (numero<8) tco.itineraires[numero].couleur(mefCol(info),true); break;
      case CODE_ERREUR:       println("ERREUR "+numero); break;
      case CODE_INFO:         println("INFO "+numero); break;
      case CODE_CAB:          if (numero<=2) cabs[numero-1].feux=info; break; // 1 ou 2
      case CODE_MEM:          println("MEM : "+((info<<8)|numero)); break; // 0 a 65535
      case CODE_TRAIN:        println("TRAIN : "+info+" sur Z"+numero); tco.boutonsTrains[numero].train(info); break; // 
      default: print('?');
    }
  } // fin draw
}

String[] noms={"XA","XB","AX","BX", "YA","YB","AY","BY"};
  
synchronized void portWrite(int c) { // pour affichage dans la console
  println();
  if (c>='0' && c<='7') print("===> itineraire "+noms[c-'0']); else
  if (c>='A' && c<='F') print("===> occuper Z"+(char)(c-'A'+'0')); else
  if (c>='a' && c<='f') print("===> liberer Z"+(char)(c-'a'+'0')); 
  print(" : ");
  port.write(c);
}

void affichagePoints() { int x,y; // pour tests
  for (Zone z : tco.zones) for (Pave p : z.paves) {
    x=p.col; y=p.lin;
    for (Forme f : p.formes) { if (f.points!=null) for (Point pt : f.points) {
      translate(B,B); // bordure
      scale(N); // zoom
      stroke(COULEUR_FOND); 
      strokeWeight(1);  
      point(x*9+pt.x,y*9+pt.y); 
      strokeWeight(1);  
      resetMatrix();        
    }} // 2!
  }
}

int mefFeu(int feux) { // transformation des feux en couleur pour signaux TCO
  if ((feux&(FEU_S|FEU_C|FEU_Cv))!=0) return COULEUR_ROUGE;
  if ((feux&(FEU_A|FEU_R|FEU_RR))!=0) return COULEUR_JAUNE;
  if ((feux&FEU_Vl)!=0)               return COULEUR_VERTE;
  return COULEUR_GRISE;
}

int mefCol(int c) { // transformation des n° en couleur
  switch (c) {
    case 0: return COULEUR_GRISE;
    case 1: return COULEUR_ROUGE;
    case 2: return COULEUR_BLANCHE;
    case 3: return COULEUR_JAUNE;
    case 4: return COULEUR_VERTE;
    case 5: return COULEUR_ROSE;
    default: return(0);
  }
}

String mefFeux(int f) { // transformation des n° en feux
  switch (f) {
    case FEU_E: return "E";
    case FEU_Vl: return "Vl";
    case FEU_A: return "A";
    case FEU_S: return "S";
    case FEU_R: return "R";
    case FEU_RR: return "RR";
    case FEU_A+FEU_RR: return "A+RR";
    case FEU_C: return "C";
    default: return "";
  }
}

void affichage(int code,int info,int numero) { // affichage lisible
  switch(code) {
    case CODE_AIGUILLE:   print('A'); break;
    case CODE_ZONE:       print('Z'); break;
    case CODE_SIGNAL:     print('S'); break;
    case CODE_CARRE:      print('C'); break;
    case CODE_BOUTON:     print('B'); break;
    case CODE_BOUTON_CLI: print('b'); break;
    case CODE_ERREUR:     return; 
    case CODE_INFO:       return; 
    case CODE_CAB:        print('s'); break;
    case CODE_MEM:        return; 
    default:              print('?');   
  }
  print(numero); 
  if (code==CODE_ERREUR) { println(); return; }
  print('='); 
  if (code==CODE_AIGUILLE) { if (info==1) print("/ "); else print("| "); return; }
   //if (code==CODE_CAB) print("***");
  if (code==CODE_CAB | code==CODE_SIGNAL | code==CODE_CARRE) { print(mefFeux(info)+" "); return; }
  else { 
    switch(info) {
      case 0:   print("gris"); break;
      case 1:   print("rouge"); break;
      case 2:   print("blanc"); break;
      case 3:   print("jaune"); break;
      case 4:   print("vert"); break;
      case 5:   print("rose"); break;
      default:  print('?');
    }  
  }
  print(' ');
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////